
/*
 * trmath.c
 *
 * This file contains miscellaneous math-only routines for the TR* viewer
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <sys/types.h>

#include "config.h"
#include "tr2.h"
#include "trmath.h"

/*
 * As the name implies, returns x**2
 */
float Square(int x)
{
   return((float)x * (float)x);
}

/*
 * As the name implies, returns the distance between (x0, y0, z0) and (x1, y1, z1)
 */
float Distance(int x0, int y0, int z0, int x1, int y1, int z1)
{
   return((float)sqrt(Square(x1 - x0) + Square(y1 - y0) + Square(z1 - z0)));
}

/*
 * Multiply two 4x4 double matrices (where the fourth column is imputed)
 */
void MatrixMultiply(double *M1, double *M2, double *Result)
{
   // 0  1  2 (0)
   // 3  4  5 (0)
   // 6  7  8 (0)
   // 9 10 11 (1)

   Result[0]  = (M1[0] * M2[0]) + (M1[1] * M2[3]) + (M1[2] * M2[6]);
   Result[1]  = (M1[0] * M2[1]) + (M1[1] * M2[4]) + (M1[2] * M2[7]);
   Result[2]  = (M1[0] * M2[2]) + (M1[1] * M2[5]) + (M1[2] * M2[8]);

   Result[3]  = (M1[3] * M2[0]) + (M1[4] * M2[3]) + (M1[5] * M2[6]);
   Result[4]  = (M1[3] * M2[1]) + (M1[4] * M2[4]) + (M1[5] * M2[7]);
   Result[5]  = (M1[3] * M2[2]) + (M1[4] * M2[5]) + (M1[5] * M2[8]);

   Result[6]  = (M1[6] * M2[0]) + (M1[7] * M2[3]) + (M1[8] * M2[6]);
   Result[7]  = (M1[6] * M2[1]) + (M1[7] * M2[4]) + (M1[8] * M2[7]);
   Result[8]  = (M1[6] * M2[2]) + (M1[7] * M2[5]) + (M1[8] * M2[8]);

   Result[9]  = M1[9] + M2[9];
   Result[10]  = M1[10] + M2[10];
   Result[11]  = M1[11] + M2[11];

   Result[9]  = (M1[9] * M2[0]) + (M1[10] * M2[0]) + (M1[11] * M2[0]) + M1[9];
   Result[10] = (M1[9] * M2[3]) + (M1[10] * M2[3]) + (M1[11] * M2[3]) + M1[10];
   Result[11] = (M1[9] * M2[6]) + (M1[10] * M2[6]) + (M1[11] * M2[6]) + M1[11];
}

/*
 * Multiply a 3-element vector by a 4x4 matrix (with column 4 imputed)
 */
void VectorMultiply(m_vector *V1, double *M2, m_vector *Result)
{
   // x
   // y
   // z

   // 0  1  2 (0)
   // 3  4  5 (0)
   // 6  7  8 (0)
   // 9 10 11 (1)

   Result->x  = (V1->x * M2[0]) + (V1->y * M2[1]) + (V1->z * M2[2]) + M2[9];
   Result->y  = (V1->x * M2[3]) + (V1->y * M2[4]) + (V1->z * M2[5]) + M2[10];
   Result->z  = (V1->x * M2[6]) + (V1->y * M2[7]) + (V1->z * M2[8]) + M2[11];
}

/*
 * Set a 4x4 matrix (with column 4 imputed) to the identity.
 */
void SetIdentity(double *matrix)
{
   int i;

   for (i = 0; i < 12; ++i)
      matrix[i] = 0.0;
   matrix[0] = 1.0;
   matrix[4] = 1.0;
   matrix[8] = 1.0;
}


